#!/bin/bash
#
# source: http://downloads.sourceforge.net/project/fontforge/fontforge-source/fontforge_full-20100501.tar.bz2
#
# maintainer: gmg (monpontet.gerard@neuf.fr)
#

#------------------Set Some Variables

soft="fontforge"
softversion="20100501"
arch="i486"
cpu="i686"
packageversion="64.1"

buildir=$(pwd)
srcpkg="$buildir/fontforge_full-20100501.tar.bz2"
src="$buildir/$soft-$softversion"
package="$soft-$softversion-$arch-$packageversion"
dest="$buildir/$package"

cd $buildir
rm -rf $src
tar -xvjf $srcpkg
rm -f $dest.tgz
rm -rf $dest

#------------------Set correct permissions

cd $src
chown -R root:root .
find . \
 \( -perm 777 -o -perm 775 -o -perm 711 -o -perm 555 -o -perm 511 \) \
 -exec chmod 755 {} \; -o \
 \( -perm 666 -o -perm 664 -o -perm 600 -o -perm 444 -o -perm 440 -o -perm 400 \) \
 -exec chmod 644 {} \;
export CFLAGS="-O2 -march=$arch -mtune=$cpu"
export CXXFLAGS="-O2 -march=$arch -mtune=$cpu"

./configure \
  --prefix=/usr \
  --mandir=/usr/man \
  --docdir=/usr/doc/$soft-$softversion \
  --disable-static

make -j4
make install DESTDIR=$dest

find $dest | xargs file | grep -e "executable" -e "shared object" | grep ELF \
  | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null

mkdir -p $dest/usr/share/applications
cp $src/Packaging/fontforge.desktop $dest/usr/share/applications
mkdir -p $dest/usr/share/icons/hicolor/32x32/apps
cp $src/Packaging/fontforge.png $dest/usr/share/icons/hicolor/32x32/apps
mkdir -p $dest/usr/share/mime/packages
cp $src/Packaging/fontforge.xml $dest/usr/share/mime/packages

find $dest/usr/man -type f -exec gzip -9 {} \;

#-------------------Prepare the documentation:

mkdir -p $dest/usr/doc/$soft-$softversion
cp -a AUTHORS INSTALL LICENSE README* VERSION $dest/usr/doc/$soft-$softversion

#-----------------Make the slack-desc files
mkdir -p $dest/install
cat << EODESC > $dest/install/slack-desc
# HOW TO EDIT THIS FILE:
# The "handy ruler" below makes it easier to edit a package description.  Line
# up the first '|' above the ':' following the base package name, and the '|'
# on the right side marks the last column you can put a character in.  You must
# make exactly 11 lines for the formatting to be correct.  It's also
# customary to leave one space after the ':'.

         |-----handy-ruler------------------------------------------------------|
fontforge: fontforge - is An outline font editor
fontforge:
fontforge: FontForge -- An outline font editor that lets you create your own
fontforge: postscript truetype, opentype, cid-keyed, multi-master, cff, svg and
fontforge: bitmap (bdf, FON, NFNT) fonts, or edit existing ones.
fontforge: 
fontforge:
fontforge:
fontforge:
fontforge:
fontforge:
EODESC

mkdir -p $dest/usr/src/$soft-$softversion
cp -a $buildir/build-$soft.sh $dest/usr/src/$soft-$softversion

#-------------------Make the packages and the md5sum

cd $dest
makepkg -l y -c n $dest.txz
makepkg -l y -c n $dest.tgz

cd $buildir
md5sum $package.txz > $package.md5